#ifndef BIDE_H_
#define BIDE_H_

#include "projDB.h"
#include "sequence.h"


class bide{
private:
	int min_sup;	
	vector<int> sdb;
	vector<int>* SCounter;
	vector<int>* SLastVisit;
	vector<int>* ICounter;
	vector<int>* ILastVisit;
	vector<int> SLFI;
	vector<int> ILFI;
	int itemAmount;
	int closedPatternCount;
	ofstream outFile;
	int longest;
	vector<int> lengthCounter;
public:
	static time_t begintime,endtime,sumtime;
	static time_t BCbtime,BCetime,BCstime;
	static time_t BSbtime,BSetime,BSstime;
	static time_t BCInbtime,BCInetime,BCInstime;
	static time_t BSInbtime,BSInetime,BSInstime;

	bide(double min_threshold,char* filename){
		sdb.push_back(-2);
		int seqCount;
		itemAmount = binaryReader(&sdb,filename,seqCount,longest)+1;
		lengthCounter.resize(longest);
		lengthCounter.assign(longest,0);
		min_sup = (int) ( (double)seqCount * (double)min_threshold);

		cout<<"Related Support: "<<min_threshold<<endl;
		cout<<"Absoluted Support: "<<min_sup<<endl;
		cout<<"Items Count: "<<itemAmount<<endl; 
		cout<<"Sequence Count: "<<seqCount<<endl;

		SCounter = new vector<int>(itemAmount,0);
		SLastVisit = new vector<int>(itemAmount,-1);
		ICounter = new vector<int>(itemAmount,0);
		ILastVisit = new vector<int>(itemAmount,-1);
		SLFI.assign(itemAmount,-1);
		ILFI.assign(itemAmount,-1);
		closedPatternCount = 0;
		sumtime = 0;
		BCstime = 0;
		BSstime = 0;
		BCInstime = 0;
		BSInstime = 0;
		outFile.open("ClosedPatterns.txt");

	}



	void L1PrefixSpan(){
		pdb rootDB;
		int LFIAmount =	rootDB.buildL1LFI(&sdb,SCounter,SLastVisit,min_sup,&SLFI);


		begintime = clock();
		vector<vector<int> >* patternVector=new vector<vector<int> >(LFIAmount);
		vector<int> patternSup(LFIAmount);
		vector<pdb*> pdbVector(LFIAmount);
		int LFIIter= 0;
		for(int i=0;i<LFIAmount;++i){
			while(SLFI[LFIIter] == -1)	++LFIIter;
			pdbVector[i] = new pdb();
			pdbVector[i]->seqBeginPos->reserve( (*SCounter)[LFIIter] );
			(*patternVector)[i].push_back(LFIIter);
			(*patternVector)[i].push_back(-1);
			patternSup[i] = (*SCounter)[LFIIter];
			++LFIIter;
		}
		
		SProjection(&rootDB,&SLFI,&pdbVector);
		//for(int i=0;i<LFIAmount;++i){//wjy
		for(i=0;i<LFIAmount;++i){
			vector<int> FPos(pdbVector[i]->seqBeginPos->size());
			vector<vector<int> > LLPos(pdbVector[i]->seqBeginPos->size());
			int patternLength = 0;
			for(int j=0;j<(*patternVector)[i].size();++j){
				if( ((*patternVector)[i])[j] == -1) ++patternLength;
			}
			//for(int j=0;j<LLPos.size();++j) LLPos[j].reserve(patternLength);//wjy
			for(j=0;j<LLPos.size();++j) LLPos[j].reserve(patternLength);
			bool check = backPruning(pdbVector[i],&(*patternVector)[i],FPos,LLPos);
			if(!check){
				for(int j=0;j<LLPos.size();++j) LLPos[j].resize(0);
			
				int _bei = backCheck(pdbVector[i],&(*patternVector)[i],FPos,LLPos);
				bidePlus(pdbVector[i],&(*patternVector)[i],patternSup[i],_bei,0);
			}
			delete pdbVector[i];
		}
		delete patternVector;
		endtime = clock();
		sumtime = endtime - begintime;
	}

	void SProjection(pdb* PDB,vector<int>* LFI,vector<pdb*>* pdbVector){
		if(pdbVector->empty()) return;
		int j;
		SLastVisit->assign(SLastVisit->size(),-1);
		for(int i=0;i<PDB->seqBeginPos->size();++i){
			j=(*PDB->seqBeginPos)[i];
			if(j>0 && sdb[j-1]>-1){
				while( sdb[j] != -1) ++j;
			}
			while(sdb[j] != -2){
				if(sdb[j] != -1){
					if( (*LFI)[ sdb[j] ] != -1 &&  (*PDB->seqBeginPos)[i]>(*SLastVisit)[sdb[j]] ){
						(*SLastVisit)[sdb[j]] = (*PDB->seqBeginPos)[i];
						(*pdbVector)[ (*LFI) [sdb[j] ] ]->seqBeginPos->push_back(j+1);
						if((*LFI) [sdb[j+1] ] == -1) (*pdbVector)[ (*LFI) [sdb[j] ] ]->seqBeginPos->back() = j+2;
					}
				}
				++j;
			}
		}
	}

	void IProjection(pdb* PDB,vector<int>* LFI,vector<int>* pattern,vector<pdb*>* pdbVector){
		if(pdbVector->empty()) return;
		int j;
		int matched = 0;
		int patternBeginPos= pattern->size()-2;
		SLastVisit->assign(SLastVisit->size(),-1);
		while(patternBeginPos>0 && (*pattern)[patternBeginPos-1] != -1 ) --patternBeginPos;
		for(int i=0;i<PDB->seqBeginPos->size();++i){
			j=(*PDB->seqBeginPos)[i];
			if(j>0 && sdb[j-1]!=-1){
				while( sdb[j] != -1){
					if( (*LFI)[ sdb[j] ] != -1 &&  (*PDB->seqBeginPos)[i]>(*SLastVisit)[sdb[j]] ){
						(*SLastVisit)[sdb[j]] = (*PDB->seqBeginPos)[i];
						(*pdbVector)[ (*LFI) [sdb[j] ] ]->seqBeginPos->push_back(j+1);
						if((*LFI) [sdb[j+1] ] == -1) (*pdbVector)[ (*LFI) [sdb[j] ] ]->seqBeginPos->back() = j+2;
					}
					++j;
				}
			}
			++j;
			while( sdb[j] != -2){
				int patternIter = patternBeginPos;
				while(sdb[j] != -1 && (*pattern)[patternIter] != -1){
					if( sdb[j] == (*pattern)[patternIter]){
						++patternIter;
						if((*pattern)[patternIter] == -1){
							matched = 1;
						}
					}
					++j;
				}
				if(matched == 0){
					while( sdb[j] != -1 ) ++j;
				}
				else{
					while( sdb[j] != -1 ) {
						if( (*LFI)[ sdb[j] ] != -1 &&  (*PDB->seqBeginPos)[i]>(*SLastVisit)[sdb[j]] ){
							(*SLastVisit)[sdb[j]] = (*PDB->seqBeginPos)[i];
							(*pdbVector)[ (*LFI) [sdb[j] ] ]->seqBeginPos->push_back(j+1);
						}
						++j;
					}
					matched = 0;
				}				
				++j;
			}
		}
	}
	
	inline int matchPatternForward(vector<int>* pattern,int& patternPos,int& seqIter){
		//btime = clock();
		int matched = 0;
		while( (*pattern)[patternPos] > -1 && sdb[seqIter] > -1){
			if( (*pattern)[patternPos] == sdb[seqIter] ){
				++patternPos;
				if( (*pattern)[patternPos] == -1){
					matched = 1;
				}
			}
			++seqIter;
		}
		//etime = clock();
		//stime += etime - btime;
		return matched;
	}

	inline int matchPatternForward(vector<int>* pattern,int& patternPos,int& seqIter,vector<int>* items,int& itemsCounter){

		int matched = 0;
		while( (*pattern)[patternPos] != -1){
			if( (*pattern)[patternPos] == sdb[seqIter] ){
				++patternPos;
				if( (*pattern)[patternPos] == -1){
					matched = 1;
				}
				++seqIter;
			}
			else{
				++seqIter;
				if(sdb[seqIter] != -1){
					return 0;
				}
				else{
					++itemsCounter;
					(*items)[itemsCounter] = sdb[seqIter];
				}
				//items->push_back(sdb[seqIter]);
			}
		}

		return matched;
	}

	inline int matchPatternBackward(vector<int>* pattern,int& patternPos,int& seqIter){
		//btime = clock();
		int matched = 0;
		//cout<<"||||"<<patternPos<<" "<<seqIter<<" "<<(*pattern)[patternPos]<<" "<<sdb[seqIter]<<endl;
		while( (*pattern)[patternPos] > -1 && sdb[seqIter] > -1){
			if( (*pattern)[patternPos] == sdb[seqIter] ){
				--patternPos;
				if( patternPos == -1 || (*pattern)[patternPos] == -1  ){
					matched = 1;
				}
			}
			--seqIter;
		}
		return matched;
	}

	int backCheck(pdb* PDB,vector<int>* pattern,vector<int>& FPos,vector<vector<int> >& LLPos){
		//sequenceDisplay(*pattern);
		int patternBegin=pattern->size()-2;
		//vector<int> FPos(PDB->seqBeginPos->size());
		//vector<vector<int> > LLPos(PDB->seqBeginPos->size());
		//int patternLength = 0;
		int patternLength = LLPos[0].capacity();
		int patternIter;
		int patternPos;
		int seqIter;
		int forwardSeqIter;
		int forwardPatternIter;
		//cancel direct member variable access
		vector<int>& sdb = this->sdb;

		//BCInbtime = clock();
		//patternBegin count back to the first item of the last event of the prefix pattern
		//BCInbtime = clock();
		//loop variable
		int i,j;
		while(patternBegin>0 && (*pattern)[patternBegin-1] != -1) --patternBegin;
		for( i=0;i<PDB->seqBeginPos->size();++i){
			seqIter = (*PDB->seqBeginPos)[i]-1;
			if(sdb[seqIter] == -1) --seqIter;
			LLPos[i].push_back(seqIter);
			patternPos = patternBegin;
			//to find find n-th last-in-last appearance of prefix sequence
			//btime = clock();
			while(sdb[seqIter] != -2){
				patternIter = patternPos;
				if(matchPatternForward(pattern,patternIter,seqIter)){
					LLPos[i].back() = seqIter-1;
				}
				//move seqIter to next event
				while(sdb[seqIter]!=-1) ++seqIter;
				++seqIter;
			}
			seqIter = LLPos[i].back();
			while(sdb[seqIter]>=0) --seqIter;
			if(sdb[seqIter] != -2){
				--seqIter;
			}
			else{
				++seqIter;
			}
			while(patternPos>=0 && (*pattern)[patternPos] != -1) --patternPos;
			--patternPos;
			while(patternPos >= 0){
				patternIter = patternPos;
				int seqIterMatchBegin =seqIter;
				if(matchPatternBackward(pattern,patternIter,seqIter)){
					LLPos[i].push_back(seqIterMatchBegin);
					while(patternPos>=0 && (*pattern)[patternPos] != -1) --patternPos;
					--patternPos;
				}
				while(sdb[seqIter]>=0) --seqIter;
				if(sdb[seqIter] != -2) {
					--seqIter;
				}
				else{
					++seqIter;
				}
			}
			while(sdb[seqIter-1] != -2) --seqIter;
			FPos[i] = seqIter;
		}

		patternPos = 0;
		int LLSize = LLPos[0].size() -1;
		int PDBSize = PDB->seqBeginPos->size() -1;
		int scounter = 0;
		int icounter = 0;
		int endIter;
		for(i=0;i<=LLSize;++i){
			SLastVisit->assign(SLastVisit->size(),-1);
			ILastVisit->assign(ILastVisit->size(),-1);
			for(j=0;j<=PDBSize;++j){
				seqIter = FPos[j];
				endIter = LLPos[j][LLSize-i];
				while(endIter >= 0 && sdb[endIter]!=-1) --endIter;
				--endIter;
				scounter = 0;
				while(seqIter<=endIter){
					if(sdb[seqIter] != -1){
						if( (*SLastVisit)[ sdb[seqIter] ] >=  j-1){
							(*SLastVisit)[ sdb[seqIter] ] = j;
							++scounter;
						}
					}
					++seqIter;
				}
				seqIter = FPos[j];
				endIter = LLPos[j][LLSize-i];
				if(sdb[endIter+1] == -2) --endIter;
				icounter = 0;

				while(seqIter<=endIter){
					patternIter = patternPos;
					forwardSeqIter = seqIter;
					forwardPatternIter = patternIter;
					if(matchPatternForward(pattern,patternIter,seqIter)){

						while( (*pattern)[forwardPatternIter] > -1){
							if( (*pattern)[forwardPatternIter] == sdb[forwardSeqIter]){
								++forwardPatternIter;
							}
							else{
								if( (*ILastVisit)[ sdb[forwardSeqIter] ] ==  j-1){
									(*ILastVisit)[ sdb[forwardSeqIter] ] = j;
									++icounter;
								}								
							}
							++forwardSeqIter;
						}
						while(sdb[seqIter] != -1 && seqIter <=endIter){
							if( (*ILastVisit)[ sdb[seqIter] ] ==  j-1){
								(*ILastVisit)[ sdb[seqIter] ] = j;
								++icounter;
							}						
							++seqIter;
						}
					}
					else{
						while(sdb[seqIter] != -1 && seqIter <=endIter) ++seqIter;
					}
					++seqIter;
				}

				patternIter = patternPos;
				seqIter = FPos[j];
				while(!matchPatternForward(pattern,patternIter,seqIter)){
					while(sdb[seqIter] != -1) ++seqIter;
					++seqIter;
					patternIter = patternPos;
				}
				while(sdb[seqIter] != -1) ++seqIter;
				++seqIter;
				FPos[j] = seqIter;
			}
			if(scounter!=0){
				return 1;
			}
			if(icounter!=0){
				return 1;
			}
			while((*pattern)[patternPos] != -1) ++patternPos;
			++patternPos;
		}
		return 0;
	}

	bool backPruning(pdb* PDB,vector<int>* pattern,vector<int>& FPos,vector<vector<int> >& LLPos){
		int patternBegin=pattern->size()-2;
		int patternLength = LLPos[0].capacity();
		int patternIter;
		int patternPos;
		int seqIter;
		int forwardSeqIter;
		int forwardPatternIter;
		vector<int>& sdb = this->sdb;

		//loop variable
		int i,j;
		int seqIterMatchBegin;

		while(patternBegin>0 && (*pattern)[patternBegin-1] != -1) --patternBegin;
		for(i=0;i<PDB->seqBeginPos->size();++i){
			seqIter = (*PDB->seqBeginPos)[i]-1;
			if(sdb[seqIter] == -1) --seqIter;
			LLPos[i].push_back(seqIter);
			patternPos = patternBegin;

			while(sdb[seqIter]>=0) --seqIter;
			if(sdb[seqIter] != -2){
				--seqIter;
			}
			else{
				++seqIter;
			}
			while(patternPos>=0 && (*pattern)[patternPos] != -1) --patternPos;
			--patternPos;
			while(patternPos >= 0){
				patternIter = patternPos;
				seqIterMatchBegin =seqIter;
				if(matchPatternBackward(pattern,patternIter,seqIter)){
					LLPos[i].push_back(seqIterMatchBegin);
					while(patternPos>=0 && (*pattern)[patternPos] != -1) --patternPos;
					--patternPos;
				}
				while(sdb[seqIter]>=0) --seqIter;
				if(sdb[seqIter] != -2) {
					--seqIter;
				}
				else{
					++seqIter;
				}

			}
			while(sdb[seqIter-1] != -2) --seqIter;
			FPos[i] = seqIter;
		}

		patternPos = 0;
		int LLSize = LLPos[0].size()-1 ;
		int PDBSize = PDB->seqBeginPos->size()-1;
		int scounter = 0;
		int icounter = 0;
		int endIter;
		for(i=0;i<=LLSize;++i){
			SLastVisit->assign(SLastVisit->size(),-1);
			ILastVisit->assign(ILastVisit->size(),-1);
			for(j=0;j<=PDBSize;++j){
				seqIter = FPos[j];
				endIter = LLPos[j][LLSize-i];
				while(endIter >= 0 && sdb[endIter]!=-1) --endIter;
				--endIter;
				scounter = 0;
				while(seqIter<=endIter){
					if(sdb[seqIter] != -1){
						if( (*SLastVisit)[ sdb[seqIter] ] >=  j-1){
							(*SLastVisit)[ sdb[seqIter] ] = j;
							++scounter;
						}
					}
					++seqIter;
				}
				seqIter = FPos[j];
				endIter = LLPos[j][LLSize-i];
				if(sdb[endIter+1] == -2) --endIter;
				icounter = 0;

				while(seqIter<=endIter){
					patternIter = patternPos;
					forwardSeqIter = seqIter;
					forwardPatternIter = patternIter;
					if(matchPatternForward(pattern,patternIter,seqIter)){

						while( (*pattern)[forwardPatternIter] > -1){
							if( (*pattern)[forwardPatternIter] == sdb[forwardSeqIter]){
								++forwardPatternIter;
							}
							else{
								if( (*ILastVisit)[ sdb[forwardSeqIter] ] ==  j-1){
									(*ILastVisit)[ sdb[forwardSeqIter] ] = j;
									++icounter;
								}								
							}
							++forwardSeqIter;
						}
						while(sdb[seqIter] != -1 && seqIter <=endIter){
							if( (*ILastVisit)[ sdb[seqIter] ] ==  j-1){
								(*ILastVisit)[ sdb[seqIter] ] = j;
								++icounter;
							}						
							++seqIter;
						}
					}
					else{
						while(sdb[seqIter] != -1 && seqIter <=endIter) ++seqIter;
					}
					++seqIter;
				}

				patternIter = patternPos;
				seqIter = FPos[j];
				while(!matchPatternForward(pattern,patternIter,seqIter)){
					while(sdb[seqIter] != -1) ++seqIter;
					++seqIter;
					patternIter = patternPos;
				}
				while(sdb[seqIter] != -1) ++seqIter;
				++seqIter;
				FPos[j] = seqIter;
			}
			if(scounter!=0){
				return true;
			}
			if(icounter!=0){

				return true;
			}
			while((*pattern)[patternPos] != -1) ++patternPos;
			++patternPos;
		}
		return false;
	}


	void bidePlus(pdb* PDB,vector<int>* pattern,int sup,int bei,int len){
		int sfei,ifei;
		int SLFIAmount = PDB->getSLFI(&sdb,SCounter,SLastVisit,min_sup,&SLFI,sup,&sfei);
		int ILFIAmount = PDB->getILFI(&sdb,ICounter,ILastVisit,min_sup,&ILFI,pattern,sup,&ifei);
		if(sfei+ifei+bei == 0){
			sequenceDisplay(*pattern,outFile);
			++lengthCounter[len];
			++closedPatternCount;
		}

		vector<vector<int> >* SPatternVector = new vector<vector<int> >(SLFIAmount,*pattern);
		vector<int> SPatternSup(SLFIAmount);
		vector<pdb*> SPdbVector(SLFIAmount);
		int SLFIIter= 0;
		for(int i=0;i<SLFIAmount;++i){
			while(SLFI[SLFIIter] == -1)	++SLFIIter;
			SPdbVector[i] = new pdb();
			SPdbVector[i]->seqBeginPos->reserve( (*SCounter)[SLFIIter] );
			(*SPatternVector)[i].push_back(SLFIIter);
			SPatternSup[i] = (*SCounter)[SLFIIter];
			(*SPatternVector)[i].push_back(-1);
			++SLFIIter;
		}

		SProjection(PDB,&SLFI,&SPdbVector);

		vector<vector<int> >* IPatternVector= new vector<vector<int> >(ILFIAmount,*pattern);
		vector<int> IPatternSup(ILFIAmount);
		vector<pdb*> IPdbVector(ILFIAmount);
		int ILFIIter= 0;
		//for(int i=0;i<ILFIAmount;++i){//wjy
		for(i=0;i<ILFIAmount;++i){
			while(ILFI[ILFIIter] == -1)	++ILFIIter;
			IPdbVector[i] = new pdb();
			IPdbVector[i]->seqBeginPos->reserve( (*ICounter)[ILFIIter]);
			(*IPatternVector)[i].back() = ILFIIter;
			IPatternSup[i] = (*ICounter)[ILFIIter];
			(*IPatternVector)[i].push_back(-1);
			++ILFIIter;
		}

		IProjection(PDB,&ILFI,pattern,&IPdbVector);


		//for(int i=0;i<SLFIAmount;++i){//wjy
		for(i=0;i<SLFIAmount;++i){
			if(SPdbVector[i]->seqBeginPos->size() == 0) break;
			vector<int> FPos(SPdbVector[i]->seqBeginPos->size());
			vector<vector<int> > LLPos(SPdbVector[i]->seqBeginPos->size());
			int patternLength = 0;
			for(int j=0;j<(*SPatternVector)[i].size();++j){
				if( ((*SPatternVector)[i])[j] == -1) ++patternLength;
			}
			//for(int j=0;j<LLPos.size();++j) LLPos[j].reserve(patternLength);//wjy
			for(j=0;j<LLPos.size();++j) LLPos[j].reserve(patternLength);

			bool check = backPruning(SPdbVector[i],&(*SPatternVector)[i],FPos,LLPos);
			if(!check){
				for(int j=0;j<LLPos.size();++j) LLPos[j].resize(0);


				int _bei = backCheck(SPdbVector[i],&(*SPatternVector)[i],FPos,LLPos);
				bidePlus(SPdbVector[i],&(*SPatternVector)[i],SPatternSup[i],_bei,len+1);
			}
			delete SPdbVector[i];
		}
		//for(int i=0;i<ILFIAmount;++i){//wjy
		for(i=0;i<ILFIAmount;++i){
			if(IPdbVector[i]->seqBeginPos->size() == 0) break;
			vector<int> FPos(IPdbVector[i]->seqBeginPos->size());
			vector<vector<int> > LLPos(IPdbVector[i]->seqBeginPos->size());
			int patternLength = 0;
			for(int j=0;j<(*IPatternVector)[i].size();++j){
				if( ((*IPatternVector)[i])[j] == -1) ++patternLength;
			}
			//for(int j=0;j<LLPos.size();++j) LLPos[j].reserve(patternLength);//wjy
			for(j=0;j<LLPos.size();++j) LLPos[j].reserve(patternLength);
			bool check = backPruning(IPdbVector[i],&(*IPatternVector)[i],FPos,LLPos);
			if(!check){
				for(int j=0;j<LLPos.size();++j) LLPos[j].resize(0);
				int _bei = backCheck(IPdbVector[i],&(*IPatternVector)[i],FPos,LLPos);
				bidePlus(IPdbVector[i],&(*IPatternVector)[i],IPatternSup[i],_bei,len+1);
			}
			delete IPdbVector[i];
		}

		delete SPatternVector;
		delete IPatternVector;
	}

	void run(){


		L1PrefixSpan();
		cout<<"Total Closed Pattens: "<<closedPatternCount<<endl;
		cout<<"BIDE Time:"<<std::setprecision(5)<<(double)(sumtime)/(double)CLK_TCK<<endl;
		cout<<"Results Length:"<<endl;
		for(int i=0;i<longest;++i){
			if(lengthCounter[i] != 0)
				cout<<i+1<<"    "<<lengthCounter[i]<<endl;
		}
		delete SCounter;
		delete SLastVisit;
		delete ICounter;
		delete ILastVisit;
		outFile.close();
	}
};
#endif