package com.ser.assist

import com.mongodb.BasicDBList
import com.mongodb.BasicDBObject
import com.mongodb.DB
import com.mongodb.DBObject
import com.mongodb.MongoClient

class FilterValidSequences {

    def threshold_values = []
    String dbName = "";
    MongoClient mongoClient = new MongoClient();
    DB db ;
    String dataPath;

    public FilterValidSequences(appName, thresholdValues, dataPath) {
        this.dbName = appName;
        db = mongoClient.getDB(appName);
        threshold_values = thresholdValues;
        this.dataPath = dataPath;
    }

    public String getMutCollectionName(threshold_value) {
        return "mut_lists" + "_" + threshold_value.replace(".", "_").replace("-", "_")
    }

    public String getDataFilePath(threshold_value) {
        return dataPath + "/" + threshold_value + "/frequent.dat" ;
    }

    public convertIndexToMethodNames(threshold_value) {
        def coll = db.getCollection("method_index");
        def f = new File(getDataFilePath(threshold_value));

        //convert index to method names
        List<String> lines = f.readLines()
        def frequentMethods =  []
        for (String l: lines) {
            List<String> items = l.split(":")[0].split(" ")

            List<String> method_names = items.inject([]) { acc, it ->
                acc + coll.find(new BasicDBObject("index", Integer.parseInt(it))).collect([]) {it}
            }

            frequentMethods.add(method_names)

        }

        return frequentMethods;
    }


    def getMethodSequences() {
        def coll = db.getCollection("method_sequence");
        return coll.find()
    }

    def getMethodTracesFromCounters(List counters) {
        BasicDBList orList = new BasicDBList()
        counters.each() {
            orList.add(new BasicDBObject("counter", it))
        }
        DBObject query = new BasicDBObject("\$or", orList)

        def coll = db.getCollection("method_trace");
        def method_traces = coll.find(query);

        method_traces.collect([]) { it }

    }

    public groupSequencesByLevels(method_trace_sequences) {

        def levels_in_sequences = []
        method_trace_sequences.each() { method_trace_seq ->
            def level = [:].withDefault {[]}
            method_trace_seq.each() { mt ->
                level[mt.get("level")] << mt
            }
            levels_in_sequences = levels_in_sequences + level.values()
        }
    }



    def getValidSequences(freq_seq, mt_at_level, acc )
    {
        if (freq_seq.size() == 0 || mt_at_level.size() == 0)
            return acc

        //println mt_at_level[0]
        String methodKey = mt_at_level[0].get("class-name") + ":" + mt_at_level[0].get("method-name")
        if ( freq_seq[0].get("method-key") == methodKey ){
            acc << mt_at_level[0]
            getValidSequences(freq_seq.drop(1), mt_at_level.drop(1), acc)
        }
        else
        {
            getValidSequences(freq_seq, mt_at_level.drop(1), acc)
        }
    }


    def populateMethodSequenceWithMethodTraces(){

        def method_sequences = getMethodSequences()
        def method_trace_sequences = method_sequences.inject([]) { acc, seq ->
            List counters = seq.get("counters")
            List method_traces =  getMethodTracesFromCounters(counters)
            //println "${counters.size()}, ${method_traces.size()}"
            acc <<  method_traces
        }

        return method_trace_sequences;

    }


    public processSequence(threshold_value){
        def frequentMethods = convertIndexToMethodNames(threshold_value)
        def method_trace_sequences = populateMethodSequenceWithMethodTraces()
        def levels_in_sequences = groupSequencesByLevels(method_trace_sequences)

        def valid_sequences = [:].withDefault {[]}
        for(def freq_seq:frequentMethods){
            for (def mt_at_level:levels_in_sequences){
                def valid_sequence = []
                getValidSequences(freq_seq, mt_at_level, valid_sequence)
                if ( valid_sequence.size() > valid_sequences[freq_seq].size() )
                    valid_sequences[freq_seq] = valid_sequence
            }
        }

        return valid_sequences;

    }

    public report(threshold_value){

        def mut_list_coll = db.getCollection(getMutCollectionName(threshold_value));
        def key = new BasicDBObject("_id", new BasicDBObject("muts", "\$muts"))
        def mm = mut_list_coll.aggregate(new BasicDBObject("\$group", key))
        def x = mm.getCommandResult()
        println x.size()

        def result = x.get("result")
        println "${dbName} : ${threshold_value}"
        result.each() {
            print it.get("_id").get("muts").size()
            print ", "
        }
        println "Total unique method traces to generate code for ${result.size()}"

    }

    public saveValidSequences(valid_sequences, threshold_value){
        def mut_list_coll = db.getCollection(getMutCollectionName(threshold_value));
        mut_list_coll.drop()

        //we will get one instance of method_trace sequence for each freq sequence
        valid_sequences.each() { key, value ->
            //create entries in mongo for this valid sequences
            //tests will be created for these sequences
            BasicDBObject o = new BasicDBObject().append("muts", value)
            mut_list_coll.insert(o)

        }
    }

    public run() {

        threshold_values.each() {
            def valid_sequences = processSequence(it);
            saveValidSequences(valid_sequences,it);
            report(it);
        }

    }

    public static void main(String[] args) {

        def base_path = "/Users/gdevanla/Documents/bide-files"
        //FilterValidSequences fvs = new FilterValidSequences("scribe", ["0.01", "0.02", "0.04", "0.08", "0.16", "0.32"], base_path + "/" + "scribe")

        //FilterValidSequences fvs = new FilterValidSequences("nanoxml", ["0.03", "0.06", "0.12"], base_path + "/" + "nanoxml");


        FilterValidSequences fvs = new FilterValidSequences("Shtutxml",
                ["4.0E-4", "8.0E-4", "0.0016", "0.0032", "0.0064", "0.0128","0.0256", "0.0512", "0.1024", "0.2048"], base_path + "/" + "Shtutxml")

        fvs.run();
    }
}
