package com.ser.assist
import com.mongodb.BasicDBObject
import com.mongodb.DB
import com.mongodb.MongoClient
import com.mongodb.util.JSON
import org.json.simple.JSONArray
import org.json.simple.JSONObject

def generateMethodIndex(String dbName) {

    MongoClient mongoClient = new MongoClient();
    DB db = mongoClient.getDB(dbName);
    def coll = db.getCollection("method_trace")

    //Get the aggregated result
    def class_name = new BasicDBObject("class-name", "\$class-name")
    class_name.put("method-name", "\$method-name")
    def key = new BasicDBObject("_id", class_name )
    def mm = coll.aggregate(new BasicDBObject("\$group", key))

    HashMap map = new HashMap()
    mm.getCommandResult().get("result").eachWithIndex { def entry, int i ->
        String mapKey = entry.get("_id").("class-name") + ":" + entry.get("_id").get("method-name")
        map.put(mapKey, i)
        println mapKey
    }

    return map
}

def saveReverseMethodMethodMapIndex(String dbName) {

    MongoClient mongoClient = new MongoClient();
    DB db = mongoClient.getDB(dbName);
    def coll = db.getCollection("method_index");
    coll.drop();

    HashMap methodIndex = generateMethodIndex(dbName)

    for ( String key : methodIndex.keySet()){
        coll.insert(new BasicDBObject("index", methodIndex.get(key)).append("method-key" , key ))
    }

}



def createMethodMapIndex(String fname, String dbName) {

    HashMap methodIndex = generateMethodIndex(dbName)

    MongoClient mongoClient = new MongoClient();
    DB db = mongoClient.getDB(dbName);
    def coll = db.getCollection("method_trace")

    //Get the aggregated result
    def class_name = new BasicDBObject("class-name", "\$class-name")
    class_name.put("method-name", "\$method-name")
    def key = new BasicDBObject("_id", class_name )
    def mm = coll.aggregate(new BasicDBObject("\$group", key))

    //Write in JSON format to file
    JSONArray list = new JSONArray()
    mm.getCommandResult().get("result").eachWithIndex { def entry, int i ->
        JSONObject map = new JSONObject()
        map.put("method-id",
                methodIndex.get(
                        entry.get("_id").get("class-name") + ":" + entry.get("_id").get("method-name")))

        map.put("method-name", entry.get("_id").get("method-name"))
        map.put("class-name", entry.get("_id").get("class-name"))
        list.add(map)
    }

    new File(fname).withWriter { out ->
        out.write(list.toJSONString())
    }

}

def getListOfTests(String dbName) {
    MongoClient mongoClient = new MongoClient();
    DB db = mongoClient.getDB(dbName);
    def coll = db.getCollection("method_trace")

    //Get the aggregated result
    def class_name = new BasicDBObject("test-class", "\$test-class")
    class_name.put("test-method", "\$test-method")
    def key = new BasicDBObject("_id", class_name )
    def mm = coll.aggregate(new BasicDBObject("\$group", key))

    List tests = new ArrayList()
    mm.getCommandResult().get("result").eachWithIndex { def entry, int i ->
        tests.add(entry.get("_id"))
    }

    return tests
}

def getMethodsForTest(String dbName, BasicDBObject test) {
    MongoClient mongoClient = new MongoClient();
    DB db = mongoClient.getDB(dbName);
    def coll = db.getCollection("method_trace")

    List methodsForTests = new ArrayList()
    coll.find(test).sort(new BasicDBObject("counter",1 )).each() {
        methodsForTests.add(it);
    }

    return methodsForTests
}




def partition(coll, Closure s) {

    def x = coll.drop(1).inject([[coll[0]]]) { acc, it ->
        if ( s(it) ) {
            acc << [it]
        }
        else
        {
            acc[-1] << it
        }

        acc
    }
}



def createSequenceFile(String dbName, String fname) {

    List tests = getListOfTests(dbName)
    HashMap methodIndex = generateMethodIndex(dbName)

    List methodsForEachTest = tests.collect([]) { BasicDBObject test ->
        def methods = getMethodsForTest(dbName, test)
        methods
    }

    def methodSequences = methodsForEachTest.inject([]) { acc,  List methods ->
        def initLevel = methods.get(0).get("level")
        List method_partitions = partition(methods) { it.get("level") == initLevel }
        //if (method_partitions.size() > 1000) { return acc }
        acc + method_partitions
    }


    methodSequences = methodSequences.inject([]) { acc1, methodSequence ->
        //eliminate repetition due to loops
        def newMethodSequence =
                methodSequence.drop(1).inject([methodSequence.get(0)]) { acc, it ->
                    if (methodIndex.get(it.get("class-name") + ":" + it.get("method-name"))
                            == methodIndex.get(acc[-1].get("class-name") + ":" + acc[-1].get("method-name")))
                    {
                        acc
                    }
                    else
                        acc << it
                }
        acc1 << newMethodSequence
    }


    methodSequences = methodSequences.findAll() {
        it.size() >= 1 && it.size < 890  //filter out calls to finalize(); better way is to check for test-index = -1  //890 for nanoxml
    }

    methodSequences.each() {
        println it.size()
    }

    println methodSequences.inject(0) { acc, it ->
            if ( it.size() > acc ) { it.size() } else acc
    }

    println ((methodSequences.inject(0) { acc, it ->
        it.size() + acc
    }) / methodSequences.size())


    JSONArray sequences = new JSONArray();
    methodSequences.eachWithIndex { List entry, int i ->
        JSONObject sequence = new JSONObject()
        sequence.put("sequence-id", i)

        //Add counters
        JSONArray counters = new JSONArray()
        counters.addAll(entry.collect([]) { it.get("counter")})
        sequence.put("counters", counters)

        JSONArray methodIds = new JSONArray()
        methodIds.addAll(
                entry.collect([]) {
                    methodIndex.get(it.get("class-name") + ":" + it.get("method-name"))
                }
        )
        sequence.put("method-id", methodIds)

        sequences.add(sequence)

    }

    new File(fname).withWriter { out ->
        out.write(sequences.toJSONString())
    }


    MongoClient mongoClient = new MongoClient();
    DB db = mongoClient.getDB(dbName);
    def coll = db.getCollection("method_sequence")
    coll.drop()

    sequences.each() {
        coll.insert(JSON.parse(it.toJSONString()))
    }

}


def generateFilesForBide(String appName, String destFolder) {
    //write the index - method_name object
   saveReverseMethodMethodMapIndex(appName)

   //Create method index
    createMethodMapIndex("${destFolder}/${appName}-method-map.json", appName)

    //Get list of tests
    createSequenceFile(appName, "${destFolder}/${appName}-sequence.json")

}

//def appName = "textdiff"
//generateFilesForBide("textdiff")

//def appName = "Shtutxml";
//def appName = "nanoxml"
//def appName =
//generateFilesForBide(appName)


def base_path = "/Users/gdevanla/Documents/bide-files"
def appNames =  ["nanoxml"] //["nanoxml"] // [ "Shtutxml"] //, "nanoxml", "scribe"]
for (def appName:appNames){
    generateFilesForBide(appName , "${base_path}/$appName")
}



//write the index - method_name object
//saveReverseMethodMethodMapIndex("nanoxml")

//Create method index
//createMethodMapIndex("/tmp/nanoxml-method-map.json", "nanoxml")

//Get list of tests
//createSequenceFile("nanoxml", "/tmp/nanoxml-sequence.json")


