package com.ser.assist
import com.mongodb.BasicDBObject
import com.mongodb.DB
import com.mongodb.DBCollection
import com.mongodb.MongoClient
import com.ser.assist.statecarver.core.StateCarverConfiguration
import com.ser.assist.testgenerator.MethodTrace
import soot.*
import soot.options.Options
import soot.toolkits.graph.ExceptionalUnitGraph
import soot.toolkits.scalar.SimpleLocalDefs

class SequenceRanker {

    String[] threshold_values;
    String dbName = "";
    MongoClient mongoClient = new MongoClient();

    public SequenceRanker(thresholdValues, String dbName ){
        this.dbName = dbName;
        this.threshold_values = thresholdValues;
    }

    public String getMutCollectionName(threshold_value) {
        return "mut_lists" + "_" + threshold_value.replace(".", "_").replace("-", "_")
    }

    public getContainerMethod(counter, level) {
        def db = mongoClient.getDB(dbName)
        def mt_coll = db.getCollection("method_trace")

        BasicDBObject query  = new BasicDBObject("level", level)
        query.append("counter",new BasicDBObject("\$lt", counter))

        def mts = mt_coll.find(query).sort(new BasicDBObject("counter", -1)).limit(1)
        return mts

    }

    public getUnitsInvokingMethod(SootMethod m, MethodTrace mt){

        List<Unit> unitsInvokingMethod = []
        Body body = m.getActiveBody();
        for (Unit u:body.getUnits()){
            if (SootHelper.doesUnitInvokeMethod(u, SootHelper.getResolvedMethod(mt.className, mt.methodName))){
                unitsInvokingMethod.add(u);
            }
        }

        return unitsInvokingMethod;
    }


    public defUseUseRelationship(MethodTrace containerMethod, List<MethodTrace> mt_seq){

        SootMethod m = SootHelper.getResolvedMethod(containerMethod.className, containerMethod.methodName);
        ExceptionalUnitGraph unitGraph = new ExceptionalUnitGraph(m.getActiveBody());
        SimpleLocalDefs simpleLocalDefs = new SimpleLocalDefs(unitGraph);

        def list_set_of_defs = []
        mt_seq.each()  { MethodTrace mt ->
            List<Unit> unitsInvokingMethod = getUnitsInvokingMethod(m, mt);
            unitsInvokingMethod.each() {
               // println "${containerMethod.className}:${containerMethod.methodName}"
                //println "${it}"
                list_set_of_defs.add(new HashSet(SootHelper.getAllDirectAndInDirectDefs(simpleLocalDefs, it)));
            }
        }

        if (list_set_of_defs.size() == 0) {
            //println "${mt_seq.get(0).className} , ${mt_seq.methodName}, ${containerMethod.className}, ${containerMethod.methodName}"
            //assert(mt_seq.size() == 0)
            return 0
        }

        HashSet defs = new HashSet(list_set_of_defs.get(0));
        for ( int i=0; i < list_set_of_defs.size() - 1; i++) {
            for ( int j=i+1; j < list_set_of_defs.size(); j++ ) {
                defs.addAll(list_set_of_defs.get(i).intersect(list_set_of_defs.get(j)));
            }
        }

       /* mt_seq.each() {
            print it.methodName;
            print ", "
        }*/
        //println  mt_seq.size() + ", " + defs.size()
        //println defs

        return defs.size();

    }

    public defUse(MethodTrace containerMethod, List<MethodTrace> mt_seq) {

        SootMethod m = SootHelper.getResolvedMethod(containerMethod.className, containerMethod.methodName);
        ExceptionalUnitGraph unitGraph = new ExceptionalUnitGraph(m.getActiveBody());
        SimpleLocalDefs simpleLocalDefs = new SimpleLocalDefs(unitGraph);

        def all_units_invoking_methods =  [] as HashSet
        mt_seq.each()  { MethodTrace mt ->
             all_units_invoking_methods.addAll(getUnitsInvokingMethod(m, mt));
            }

        //println "${containerMethod.className} , ${containerMethod.methodName}"
        if ( all_units_invoking_methods.size() < mt_seq.size() && mt_seq.size() != 1){
            println "Warning..${containerMethod.className}:${containerMethod.methodName} is invalid"
        }

        def defUseStatements = [] as Set
        mt_seq.each() {
            MethodTrace mt ->
                List<Unit> unitsInvokingMethod = getUnitsInvokingMethod(m, mt);
                unitsInvokingMethod.each() {
                    defUseStatements.addAll(all_units_invoking_methods.intersect(
                            SootHelper.getAllDirectAndInDirectDefs(simpleLocalDefs, it)));
                }
        }

       /* print "${containerMethod.className} : ${containerMethod.methodName} "
        mt_seq.each() {
            print "${it.methodName} "
        }
        println "${mt_seq.size()} , ${defUseStatements.size()}"*/
        return defUseStatements.size()

    }

    public void rank() {

        //We do the same set of processing for sequences obtained for each
        //threshold value
        threshold_values.each() {

            //Clean up db, so to make this class idempotent
            DB db = mongoClient.getDB(dbName)
            DBCollection coll = db.getCollection(getMutCollectionName(it) + "_ranked");
            coll.drop()

            def  mut_lists = StateStore.getCollatedMethodTraces(dbName, getMutCollectionName(it));

            //println "Size of muts ${mut_lists.size()}"

            mut_lists.each() {  List<MethodTrace> mt_list->
                rankSequence(mt_list, it)
            }

        }




    }

    public void rankSequence(List<MethodTrace> mt_list, threshold_value) {
        if (mt_list.size() ==0){
            return
        }

        int level = mt_list.get(0).level
        int counter = mt_list.get(0).counter

        //level = 35
        //counter = 181

        def mt = getContainerMethod(counter, level - 1);
        def defUseUseCount = 0
        def defUseCount = 0

        if (mt.size() == 0) {
            defUseUseCount = 1
            defUseCount = 1
        }
        else {
            def containerMethod = mt.next();
            MethodTrace containerMethodTrace = new MethodTrace(containerMethod["counter"],
                    containerMethod["class-name"],
                    containerMethod["method-name"], containerMethod["level"], containerMethod["object-id"])

            defUseUseCount = defUseUseRelationship(containerMethodTrace, mt_list);
            if ( mt_list.size() == 1 && defUseUseCount == 0) defUseUseCount = 1 //reset for sequence with just one method.

            defUseCount = defUse(containerMethodTrace, mt_list);
        }

        println "counts: ${mt_list.size()} , ${defUseCount}, ${defUseUseCount}"
        saveRankedSequence(mt_list, defUseUseCount, defUseCount, threshold_value)

    }

    public saveRankedSequence(List<MethodTrace> mt_list, int defUseUseCount, int defUseCount, threshold_value) {
        DB db = mongoClient.getDB(dbName)
        DBCollection coll = db.getCollection(getMutCollectionName(threshold_value) + "_ranked");

        def muts = []
        mt_list.each() {
            muts.add(StateStore.getMethodTraceByCounter(dbName, it.counter));
        }
        BasicDBObject dbObject = new BasicDBObject("muts", muts).append("defUseUse", defUseUseCount ).append("defUse", defUseCount)
        coll.insert(dbObject)
    }

    public static void main(String[] args){

        //AssistConfiguration.config_properties_fname = "assist.properties"
        //AssistConfiguration.config_properties_fname = "jconsole.properties"
        AssistConfiguration.config_properties_fname = "scribe.properties"
        //AssistConfiguration.config_properties_fname = "jsonrpc.properties"
        //AssistConfiguration.config_properties_fname = "jagg.properties"
        //AssistConfiguration.config_properties_fname = "shtutxml.properties"
        //AssistConfiguration.config_properties_fname = "nanoxml.properties"


        String appName = "scribe";
        String[] threshold_values = ["0.01", "0.02", "0.04", "0.08", "0.16", "0.32"];

        //String appName = "nanoxml"
        //String[] threshold_values =  ["0.03", "0.06", "0.12"]


        //String appName = "Shtutxml"
        //String[] threshold_values =  ["4.0E-4", "8.0E-4", "0.0016", "0.0032", "0.0064", "0.0128","0.0256", "0.0512", "0.1024", "0.2048"]


        final SequenceRanker c = new SequenceRanker( threshold_values , appName)

        //Collect the oracles
        setSootOptions(c);

        String sourceFolder = StateCarverConfiguration.v().getProcessDir();
        String sootClassPath = StateCarverConfiguration.v().getSootClassPath();
        String[] sootArguments = ["-process-dir", sourceFolder, "-cp", sootClassPath];


        soot.Main.main(sootArguments)
       // c.rank();

    }

    def static setSootOptions(SequenceRanker r) {
        G.reset();
        //Options.v().set_verbose(true);
        Options.v().set_no_bodies_for_excluded(true);
        ArrayList<String> exclude_list = new ArrayList<String>();
        exclude_list.add("java.");
        Options.v().set_exclude(exclude_list);
        Options.v().set_whole_program(true);
        Options.v().set_output_format(Options.output_format_jimple);
        Options.v().set_keep_line_number(true);
        Options.v().setPhaseOption("jb", "use-original-names");
        //Options.v().set_output_dir(.v().getIntgTestDest());
        //Options.v().set_verbose(verbose);
        //Options.v().set_output_format(outputFormat);
        Options.v().set_allow_phantom_refs(true)


        PackManager.v().getPack("wjtp").add(new Transform("wjtp.myTransformer", new SceneTransformer() {
            @Override
            protected void internalTransform(String s, Map<String, String> stringStringMap) {
                  r.rank();
            }
        }));
    }



}
