# Instructions for using ASSIST

## Dependencies

1. SOOT framework. It can be got from https://github.com/Sable/soot
2. MongoDB server. It can be installed locally
3. XStream library from Thoughtworks.
4. BIDE sequence mining library. Available in the sequence_mining folder with the current package. The BIDE executable is only compatible with Windows.
5. All other standard libraries included in the pom.xml are available in the maven repositories.

# How to use the application

1. Make the following changes to assist.properties file
   a. global.assist_classpath = variable points to the current applications jar file, before the applicaiton is executed.
   b. global.xstreamjar = classpath of XStream jar library from Thoughtworks
   c. global.jcejar = path to local jce.jar
   d. global.junitjar = path to local junit 4 jar.
   e. aut.source_folder = source folder of subject app
   f. aut.test_source_folder = source folder of subject apps tests
   g. aut.app_class_prefix = the class files in the app that need to be instrumented
   h. aut.app_name = unique name that will be used to create a database in the mongodb database.
   i. assist.trace_destination = temp folder
   j. assist.soot.output_folder = folder where soot output files will be generated
   k. assist.integration_test_destination =  folder where integration tests will be generated and copied
   l. assist.integration_tests_package = package name that the integration tests need to be created under.

2. Second step is to instrument the application. A subject app can be instrumented using the code shown in
   com.ser.assist.tests.statecarver.TestStateCarver.

3. Once the code is instrumented, using a terminal window, `mvn test` can be executed from the instrumented folder. This step runs the application and writes all execution trace information to the mongodb server. Also, make sure the StateCarver.jar file is included as a dependency in the ant or pom file that comes with the subject app.

4. Running Sequence Mining:
   a. To prepare data for sequence mining, the MongoAnalysis.groovy needs to be run after changing the following variables:
   i. base_path = where the bide files need to be created.
   ii. appNames = The database name in mongodb that needs to be used.

   By, running this script all data needed for sequence mining would be written to a file.

5. The sequence mining step is performed by running using the tool provided in the sequence_mining folder. Please folow the readme.md under `sequence_mining` folder to perform this step. Once all steps are performed then continue with Step 6 as described in this document.

6. Then, CollateMethodTraces.groovy will read the sequence mining output and create respective entries in MongoDB databbase. The variables, `dbName` and `frequentDatFilePath` need to be appropriately set.

7. Test Generation process: Next step in the process would be to generate integration tests. This is performed by executing com.ser.assist.testgenerator.Core. Please make sure the variable AssistConfiguration.config_properties_fname is set to the correct configuration file in the main() function. This step will produce integration tests in the folder referenced by the variable assist.integration_test_destination.

8. The generated tests can be integrated into the main project and the tests can be run. The required dependencies will have to be added to the appropriate build files.

Steps 3, 4, 6 and 7 the mongodb server is running. This is necessary for saving execution traces and state data, retrieving them for preparing sequences for mining, for test generation and execution of generated tests.
