package net.n3.nanoxml;

import java.io.StringWriter;
import java.lang.reflect.Field;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;

import static junit.framework.Assert.*;

/**
 * Created by gdevanla on 3/17/14.
 */
public class Main {

    private IXMLParser target = null;

    private StdXMLBuilder builder = null;
    private IXMLReader reader = null;
    private NonValidator validator=null;



    public static void main(String[] args) throws Exception {

        Main m = new Main();
        m.testAnalyzeDataChildrenCount();
        m.testAnalyzeDataRootElement();
        m.testGenerateXML();
        m.testNamespaceAccess();
        m.testNamespaceCreation();
        m.testSimpleXML();
        m.setUp();

        m.testEndElement();
        m.testParseCDATA();
        m.testParseChildNull();
        m.testParseEntity();
        m.testParseEntityIgnoreSection();
        m.testParseFixedAttribute();
        m.testParseGetAllAttribute();
        m.testParseIgnoreSection();
        m.testParseRequiredAttribute();
        m.testParseInternalDTDSkipComment();
        m.testParsePublicNonValidator();
        m.testParseWithAttXMLNS();
        m.testParserInternalDocTypeWithNameSpace();
        m.testPublicID();
        m.testParseIncludeSection();
        m.testParseInternalDTD();
        m.testParseSkipComment();
        m.testParserWithAttDTD();
        m.testParserWithAttNoDTD();
        m.testParserWithNS();
        m.testParserWithOutNS();
        m.testprivateFields();





    }

    public void testSimpleXML()   throws Exception
    {
        String inputData = "<FOO>\n" +
                "    <BAR/>\n" +
                "    <BAR x='1'/>\n" +
                "    <BAR x='1' y='2'/>\n" +
                "    <BAR>\n" +
                "        <BAR/>\n" +
                "        <BARBAR/>\n" +
                "        <BA/>\n" +
                "        <FOO/>\n" +
                "    </BAR>\n" +
                "    <BAR>blah</BAR>\n" +
                "    <BAR x='1&lt;'>blah</BAR>\n" +
                "    <BAR><![CDATA[<&>'\"]]></BAR>\n" +
                "    <BAR>a<![CDATA[b]]>c</BAR>\n" +
                "        &#60;&#9;&#10;&#13;&lt;&amp;&gt;&quot;&apos;&#33;&#x3A;&#x2030;\n" +
                "</FOO>";

        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();
        (new XMLWriter(System.out)).write(xml, false);

    }

    public void testAnalyzeDataRootElement()   throws Exception
    {

        String inputData = "<FOO attr1=\"fred\" attr2=\"barney\">\n" +
                "    <BAR a1=\"flintstone\" a2=\"rubble\">\n" +
                "        Some data.\n" +
                "    </BAR>\n" +
                "    <QUUX/>\n" +
                "</FOO>";


        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();

        Properties props = xml.getAttributes();
        System.out.println(props.getProperty("attr1"));
        assertEquals(props.getProperty("attr1"), "fred");
        assertEquals(xml.getName(), "FOO");

    }

    public void testAnalyzeDataChildrenCount()   throws Exception
    {

        String inputData = "<FOO attr1=\"fred\" attr2=\"barney\">\n" +
                "    <BAR a1=\"flintstone\" a2=\"rubble\">\n" +
                "        Some data.\n" +
                "    </BAR>\n" +
                "    <QUUX/>\n" +
                "</FOO>";

        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();

        Properties props = xml.getAttributes();
        assertEquals(xml.getChildrenCount(), 2);
        assertEquals(xml.getChildAtIndex(0).getName(), "BAR");
        assertEquals(xml.getChildAtIndex(1).getName(), "QUUX");

    }

    public void testGenerateXML() throws Exception {

        StringWriter writer = new StringWriter();
        IXMLElement xmltree = new XMLElement("RootElement");
        xmltree.setAttribute("rootattr1", "rootattr1-value");
        IXMLElement xmlChild = new XMLElement("BAR");
        xmlChild.setAttribute("foo-attr", "foo-attr-value");
        xmltree.addChild(xmlChild);

        XMLWriter xmlWriter = new XMLWriter(writer);
        xmlWriter.write(xmltree);

        assertEquals(writer.toString(),
                "<RootElement rootattr1=\"rootattr1-value\"><BAR foo-attr=\"foo-attr-value\"/></RootElement>");

    }

    public void testNamespaceCreation() throws Exception {

        StringWriter writer = new StringWriter();
        IXMLElement xmltree = new XMLElement("Book");
        xmltree.setAttribute("title", "Answer To Life");
        xmltree.setName("Book", "xmlns:doc=\"http://nanoxml.n3.net/book");
        IXMLElement xmlChild = new XMLElement("Chapter");
        xmlChild.setAttribute("no", "1");
        xmltree.addChild(xmlChild);

        XMLWriter xmlWriter = new XMLWriter(writer);
        xmlWriter.write(xmltree);

        assertEquals(writer.toString(),
                "<Book xmlns=\"xmlns:doc=\"http://nanoxml.n3.net/book\" title=\"Answer To Life\"><Chapter no=\"1\"/></Book>");

    }

    public void testNamespaceAccess() throws Exception {

        String inputData = "<doc:Book xmlns:doc=\"http://nanoxml.n3.net/book\" title=\"Answer To Life\"><Chapter no=\"1\"/></doc:Book>";

        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();
        assertEquals(xml.getFullName(), "doc:Book");

    }


    public void setUp() throws Exception {

        target = XMLParserFactory.createDefaultXMLParser();
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        builder = new StdXMLBuilder();
        target.setBuilder(new StdXMLBuilder());
        validator=new NonValidator();
        target.setValidator(validator);
    }

    private Object getField(Object instance, String fieldName) throws Exception {
        Class c = instance.getClass();

        // Retrieve the field with the specified name
        Field f = c.getDeclaredField(fieldName);
        f.setAccessible(true);

        // Return the value of the field for the instance
        return f.get(instance);
    }


    public void testprivateFields() throws Exception {
        IXMLBuilder builder = (IXMLBuilder) getField(target, "builder");
        IXMLReader reader = (IXMLReader) getField(target, "reader");
        IXMLEntityResolver entityResolver = (IXMLEntityResolver) getField(
                target, "entityResolver");
        IXMLValidator validator = (IXMLValidator) getField(target, "validator");
        Hashtable entities = (Hashtable) getField(entityResolver, "entities");
        assertEquals(5, entities.size());

    }

    /*
	(expected = XMLParseException.class)
	public void testParserUnExpectedEntity() throws Exception {
		try {
			IXMLReader reader = StdXMLReader
					.fileReader("TestInputFiles/unexpectedEntityError.xml");
			target.setReader(reader);
			IXMLElement element = new XMLElement("note",
					"file:unexpectedEntityError.xml", 1);
			builder = new StdXMLBuilder(element);
			target.setBuilder(builder);
			target.setValidator(new NonValidator());
			target.parse();
		} catch (XMLParseException e) {
			System.out.println(e.getMessage());
			throw e;
		}
	}

    /*
	(expected = XMLParseException.class)
	public void testException() throws Exception {
		try {
			IXMLReader reader = StdXMLReader.fileReader("TestInputFiles/cData.xml");
			target.setReader(reader);
			target.parse();
			assertTrue(target.parse() instanceof XMLElement);
		} catch (XMLParseException e) {
			System.out.println(e.getMessage());
			throw e;
		}

	} */


    public void testParserWithOutNS() throws Exception {
        reader = StdXMLReader.fileReader("src/test/resources/main.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("file:src/test/resources/Note.dtd", xml.getSystemID());
        assertEquals("note", xml.getName());
        assertEquals("note", xml.getFullName());
        assertEquals(4, xml.getChildrenCount());
        assertEquals("to", xml.getChildAtIndex(0).getName());
        assertEquals("from", xml.getChildAtIndex(1).getName());
        assertEquals("heading", xml.getChildAtIndex(2).getName());
        assertEquals("body", xml.getChildAtIndex(3).getName());
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParserWithNS() throws Exception {
        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/MainWithNS.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("file:src/test/resources/MainWithNS.xml", xml.getSystemID());
        assertEquals(2, xml.getChildrenCount());
        assertEquals("root", xml.getName());

        children = xml.getChildAtIndex(0);
        assertEquals("http://www.w3.org/TR/html4/", children.getNamespace());
        assertEquals("h:table", children.getFullName());

        assertEquals("h:tr", children.getChildAtIndex(0).getFullName());
        assertEquals("h:td", children.getChildAtIndex(0).getChildAtIndex(0)
                .getFullName());

        children = xml.getChildAtIndex(1);
        assertEquals("http://www.w3schools.com/furniture",
                children.getNamespace());
        assertEquals("f:table", children.getFullName());
        assertEquals("f:name", children.getChildAtIndex(0).getFullName());
        assertEquals("f:width", children.getChildAtIndex(1).getFullName());
        assertEquals("f:length", children.getChildAtIndex(2).getFullName());
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParserWithAttNoDTD() throws Exception {
        reader = StdXMLReader.fileReader("src/test/resources/AttNoDTD.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("person", xml.getName());
        assertEquals("female", xml.getAttribute("sex", "male"));
        Vector v = xml.getChildrenNamed("firstname");
        IXMLElement name = (IXMLElement) v.get(0);
        assertEquals("firstname", name.getName());
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParserWithAttDTD() throws Exception {
        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/MainWithAtt.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();


        assertEquals("note", xml.getName());
        children = xml.getChildAtIndex(2);
        assertFalse(children.hasAttribute("tag"));
        assertEquals("h1", children.getAttribute("tag", "h1"));
        children = xml.getChildAtIndex(3);
        assertTrue(children.hasAttribute("tag"));
        assertEquals("content", children.getAttribute("tag", "h1"));
        assertNull(children.getAttributeNamespace("tag"));
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }




    public void testParseSkipComment() throws Exception {
        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/MainWithAtt.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("note", xml.getName());
        children = xml.getChildAtIndex(0);
        // this is inside comment so it skips everything in the comment
        assertNotSame("content", children.getName());
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParseInternalDTD() throws Exception {

        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("file:src/test/resources/mainInternal.xml", xml.getSystemID());
        assertEquals("note", xml.getName());
        assertEquals("note", xml.getFullName());
        assertEquals(5, xml.getChildrenCount());
        assertEquals("to", xml.getChildAtIndex(0).getName());
        assertEquals("from", xml.getChildAtIndex(1).getName());
        assertEquals("heading", xml.getChildAtIndex(2).getName());
        assertEquals("body", xml.getChildAtIndex(3).getName());
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParseInternalDTDSkipComment() throws Exception {

        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("note", xml.getName());
        children = xml.getChildAtIndex(2);
        assertFalse(children.hasAttribute("tag"));
        // skips the ATTLIST in dtd since it is inside comment
        assertNotSame("h1", children.getAttribute("tag"));
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParseCDATA() throws Exception {

        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("note", xml.getName());
        children = xml.getChildAtIndex(4);

        assertEquals("script", children.getName());
        assertFalse(children.hasChildren());
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);

    }


    public void testParseIgnoreSection() throws Exception {

        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("note", xml.getName());
        assertFalse(xml.hasAttribute("tag"));
        // skips the ATTLIST in dtd since it is inside Ignore Section
        assertNotSame("title", xml.getAttribute("tag"));
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParseIncludeSection() throws Exception {
        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("note", xml.getName());

        // Include section has attribute for child 1

        children = xml.getChildAtIndex(1);
        assertEquals("yes", children.getAttribute("include"));
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }

    // Entity Type decarations

    public void testParseEntity() throws Exception {
        IXMLElement children = null;
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        // Child 3 body has entity type content defined in xml document
        children = xml.getChildAtIndex(3);
        assertEquals("this is content with proper text", children.getContent());
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }


    public void testParseEntityIgnoreSection() throws Exception {
        IXMLElement children = null;
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("note", xml.getName());
        children = xml.getChildAtIndex(0);

        // Entity has Ignore Section
        assertFalse(children.hasAttribute("tag"));
        assertNotSame("title", children.getAttribute("tag"));
        assertEquals("toAddress", children.getAttribute("title"));
        XMLWriter writer = new XMLWriter(System.out);
        writer.write(xml);
    }

    // Public Identifier in the DocType

    public void testPublicID() throws Exception {
        reader = StdXMLReader.fileReader("src/test/resources/MainWithAtt.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("file:src/test/resources/example.dtd", xml.getSystemID());
    }


    public void testEndElement() throws Exception {
        reader = StdXMLReader.fileReader("src/test/resources/MainWithAtt.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals(5, xml.getChildrenCount());
    }

    /*
	(expected = XMLParseException.class)
	public void testImProperEndElement() throws Exception {
		try {
			reader = StdXMLReader.fileReader("src/test/resources/inValid.xml");
			target.setReader(reader);
			IXMLElement xml = (IXMLElement) target.parse();
		} catch (XMLParseException e) {
			System.out.println(e.getMessage());
			throw e;
		}
	}  */


    public void testParserInternalDocTypeWithNameSpace() throws Exception {
        reader = StdXMLReader.fileReader("src/test/resources/sample.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals(1, xml.getChildrenCount());
        assertEquals("http://www.xmlpowercorp.com/dtds/", xml.getNamespace());
    }

    /*
	(expected = XMLParseException.class)
	public void testParserWithInValidCloseTag() throws Exception {
		try {
			reader = StdXMLReader.fileReader("src/test/resources/ErrorInput.xml");
			target.setReader(reader);
			IXMLElement xml = (IXMLElement) target.parse();
		} catch (XMLParseException e) {
			System.out.println(e.getMessage());
			throw e;
		}
	}

	(expected = XMLParseException.class)
	public void testParserErrorCloseTag() throws Exception {
		try {
			reader = StdXMLReader.fileReader("src/test/resources/errorClose.xml");
			target.setReader(reader);
			IXMLElement xml = (IXMLElement) target.parse();
		} catch (XMLParseException e) {
			System.out.println(e.getMessage());
			throw e;
		}
	}  */

    // Process the entity which has a xmlns as attribute

    public void testParseWithAttXMLNS() throws Exception {
        reader = StdXMLReader.fileReader("src/test/resources/AttNoDTD.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        assertEquals("person", xml.getName());
        assertEquals("http://www.w3.org/TR/html4/", xml.getNamespace());
    }


    public void testParseChildNull() throws Exception {
        reader = StdXMLReader.fileReader("src/test/resources/childNull.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        IXMLElement child = new XMLElement(null);
        xml.addChild(child);
        assertNull(child.getName());
        assertEquals("person", xml.getName());
    }

    /*
	(expected=IllegalArgumentException.class)
	public void testParseIllegalChild() throws Exception {
		reader = StdXMLReader.fileReader("src/test/resources/childNull.xml");
		target.setReader(reader);
		IXMLElement xml = (IXMLElement) target.parse();
		xml.addChild(null);
	}       */


    //parses an Fixed Attribute for an element

    public void testParseFixedAttribute() throws Exception
    {
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();

    }

    //parses an required Attribute for an element

    public void testParseRequiredAttribute() throws Exception
    {
        reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();

    }

    /*
	public void testParseEnumeratedAttribute() throws Exception
	{
		reader = StdXMLReader.fileReader("src/test/resources/mainInternal.xml");
		target.setReader(reader);
		IXMLElement xml = (IXMLElement) target.parse();
		IXMLElement headingChild=xml.getChildAtIndex(2);
		assertEquals("h2",headingChild.getAttribute("markUp"));
		Enumeration attributeName=headingChild._enumerateAttributeNames();
		while(attributeName.hasMoreElements())
		{
			assertEquals("markUp",attributeName.nextElement());
		}
	}   */


    public void testParseGetAllAttribute() throws Exception
    {
        reader = StdXMLReader.fileReader("src/test/resources/MainWithAtt.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();
        IXMLElement bodyChild=xml.getChildAtIndex(3);

        Properties allAttributes=bodyChild.getAttributes();

    }

    // Checks the validator public and system id

    public void testParsePublicNonValidator() throws Exception
    {
        reader = StdXMLReader.fileReader("src/test/resources/ValidatorPubSys.xml");
        target.setReader(reader);
        IXMLElement xml = (IXMLElement) target.parse();

        Hashtable entities=(Hashtable)getField(validator.parameterEntityResolver,"entities");
        String[] str=(String[])entities.get("student");
        assertEquals("pubID",(str[0]));
        assertEquals("example.dtd",(str[1]));
    }

}
