package net.n3.nanoxml;


import junit.framework.TestCase;

import java.util.Stack;
import java.util.Vector;


public class StdXMLBuilderTest  extends TestCase {

	public void testStdBuilderNullElement() throws Exception
	{
        StdXMLBuilder target=new StdXMLBuilder();
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		assertNull(stackVar);
		IXMLElement root=(IXMLElement)NanoXMLUtil.getField(target,"root");
		assertNull(root);
		IXMLElement prototype=(IXMLElement)NanoXMLUtil.getField(target,"prototype");
		assertNull(prototype.getName());
	}
	

	public void testStdBuilderElement() throws Exception
	{
        StdXMLBuilder target=new StdXMLBuilder();
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		assertNull(stackVar);
		IXMLElement root=(IXMLElement)NanoXMLUtil.getField(target,"root");
		assertNull(root);
		IXMLElement prototype=(IXMLElement)NanoXMLUtil.getField(target,"prototype");
		assertEquals("note",prototype.getName());
	}
	

	public void testStartBuilding() throws Exception
	{
        StdXMLBuilder target=new StdXMLBuilder();
		target.startBuilding("file:insert.xml",1);
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		assertNotNull(stackVar);
		IXMLElement root=(IXMLElement)NanoXMLUtil.getField(target,"root");
		assertNull(root);
		
	}
	

	public void testStartElementStackEmpty() throws Exception
	{
        StdXMLBuilder target=new StdXMLBuilder();
		target.startBuilding("file:sample.xml",1);
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		assertTrue(stackVar.empty());
		target.startElement("book","f","http://www.w3c.org/html4","file:sample.xml",1);
		
		IXMLElement stackTop=(IXMLElement) stackVar.peek();
		assertEquals("book",stackTop.getName());
		assertFalse(stackVar.empty());
		IXMLElement root=(IXMLElement)NanoXMLUtil.getField(target,"root");
		assertEquals("book",root.getName());
		IXMLElement prototype=(IXMLElement)NanoXMLUtil.getField(target,"prototype");
		assertFalse(prototype.hasChildren());
		assertEquals(root,target.getResult());
	}
	
	
	// insert a new element 
	//when stack is not empty
	// so new element becomes the new stack Top
	//and checks whether new element is the child of the old stack top

	public void testStartElementStackNotEmpty() throws Exception
	{
        StdXMLBuilder target=new StdXMLBuilder();
		target.startBuilding("file:sample.xml",1);
		target.startElement("book","f","http://www.w3c.org/html4","file:sample.xml",1);
		
		IXMLElement root=(IXMLElement)NanoXMLUtil.getField(target,"root");
		assertEquals("book",root.getName());
		
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		IXMLElement stackTop=(IXMLElement) stackVar.peek();
		assertEquals("book",stackTop.getName());
		assertEquals("f:book",stackTop.getFullName());
		
		target.startElement("noteBook","f","http://www.w3c.org/html4","file:sample.xml",1);
		IXMLElement newStackTop=(IXMLElement) stackVar.peek();
		assertEquals("noteBook",newStackTop.getName());
		assertFalse(stackVar.empty());
		assertTrue(stackTop.hasChildren());
		assertEquals("f:noteBook",newStackTop.getFullName());
		Vector children=stackTop.getChildren();
		assertTrue(children.contains(newStackTop));
	}
	

	public void testEndElement() throws Exception
	{
        StdXMLBuilder target=new StdXMLBuilder();
		target.startBuilding("file:sample.xml",1);
		target.startElement("note","f","http://www.w3c.org/html4","file:sample.xml",1);
		target.startElement("book","f","http://www.w3c.org/html4","file:sample.xml",1);
		target.startElement(null,"f","http://www.w3c.org/html4","file:sample.xml",1);
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		target.endElement("noteBook1", "f:","http://www.w3c.org/html4");
		IXMLElement stackTop=(IXMLElement)stackVar.peek();
		target.endElement("noteBook2", "f:","http://www.w3c.org/html4");
		assertEquals(1,stackTop.getChildrenCount());
		
	}
	
	//checks without Adding Attribute and adding attribute

	public void testAddAtrributeWithNameSpace() throws Exception
	{
        StdXMLBuilder target=new StdXMLBuilder();
		target.startBuilding("file:sample.xml",1);
		target.startElement("note","f","http://www.w3c.org/html4","file:sample.xml",1);
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		IXMLElement stackTop=(IXMLElement)stackVar.peek();
		assertFalse(stackTop.hasAttribute("f:name"));
		target.addAttribute("name","f","http://www.w3c.org/html4","Software Testing","String");
		stackTop=(IXMLElement)stackVar.peek();
		assertTrue(stackTop.hasAttribute("f:name"));
		
	}
	

	public void testAddAtrributeWithOutNameSpace() throws Exception
	{

        StdXMLBuilder target=new StdXMLBuilder();
		target.startBuilding("file:sample.xml",1);
		target.startElement("note",null,null,"file:sample.xml",1);
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		IXMLElement stackTop=(IXMLElement)stackVar.peek();
		assertFalse(stackTop.hasAttribute("f:name"));
		target.addAttribute("name",null,null,"Software Testing","String");
		stackTop=(IXMLElement)stackVar.peek();
		assertTrue(stackTop.hasAttribute("name"));
		
	}

    /*
	(expected=XMLParseException.class)
	public void testAddAtrribute() throws Exception
	{
		target.startBuilding("file:sample.xml",1);
		target.startElement("note",null,null,"file:sample.xml",1);
		Stack stackVar=(Stack)NanoXMLUtil.getField(target,"stack");
		IXMLElement stackTop=(IXMLElement)stackVar.peek();
		assertFalse(stackTop.hasAttribute("f:name"));
		target.addAttribute("name",null,null,"Software Testing","String");
		target.addAttribute("name",null,null,"Software Testing","String");
		fail("should throw XMLParseException");
	} */
}
