package net.n3.nanoxml;


import junit.framework.TestCase;

import java.io.FileInputStream;
import java.io.IOException;


public class StdXMLReaderTest extends TestCase {

	public void testValidEncoding() throws Exception
	{
        StdXMLReader target=new StdXMLReader(new FileInputStream("src/test/resources/test.xml"));
        String encoding=target.getEncoding("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
		assertEquals("ISO-8859-1",encoding);
	}

	public void testEncodingNull() throws Exception
	{
        StdXMLReader target=new StdXMLReader(new FileInputStream("src/test/resources/test.xml"));
        String encoding=target.getEncoding("<?version=\"1.0\" encoding=\"ISO-8859-1\"?>");
		assertNull(encoding);
		encoding=target.getEncoding("<?xml?>");		
		assertNull(encoding);
		encoding=target.getEncoding("<?xml version=\"1.0\" encoding23=\"ISO-8859-1\"?>");
		assertNull(encoding);
		encoding=target.getEncoding("<?xml");		
		assertNull(encoding);
	}
	
	
	public void testStringReader() throws IOException {

        StdXMLReader target=new StdXMLReader(new FileInputStream("src/test/resources/test.xml"));
        String str = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\r\n" +
				"<!DOCTYPE note SYSTEM \"Note.dtd\">\r\n" + 
				"<note>\r\n" + 
				"<to>Tove</to>\r\n" + 
				"<from>Jani</from>\r\n" + 
				"<heading>Reminder</heading>\r\n" + 
				"<body>Don't forget me this weekend!</body>\r\n" + 
				"</note>";
		assertTrue(target.stringReader(str) instanceof StdXMLReader);
	}
	
	
	public void testFileReader() throws Exception
	{
        StdXMLReader target=new StdXMLReader(new FileInputStream("src/test/resources/test.xml"));

        IXMLReader r=target.fileReader("src/test/resources/test.xml");
		assertEquals(1,r.getLineNr());
		assertEquals("file:src/test/resources/test.xml",r.getSystemID());
		assertEquals("",r.getPublicID());
		assertFalse(r.atEOF());
		assertEquals(0,r.getStreamLevel());
	}

    /*
	(expected = FileNotFoundException.class)
	public void testFileReaderFileNotFound() throws Exception
	{		
			IXMLReader r=target.fileReader("src/test/resources/FileNotFound.xml");
	} */
	
	
	public void testReaderPublicIDandSystemID() throws Exception
	{
        StdXMLReader target=new StdXMLReader(new FileInputStream("src/test/resources/test.xml"));
        target=new StdXMLReader(null,"src/test/resources/encoding.xml");
		assertEquals("file:src/test/resources/encoding.xml",target.getSystemID());
		assertNull(target.getPublicID());
		assertEquals(0,target.getStreamLevel());
		assertEquals(1,target.getLineNr());
				
	}

    /*
	(expected=FileNotFoundException.class)
	public void testFileNotFound() throws Exception
	{
		target=new StdXMLReader(null,"E://encoding.xml");
		fail("Should throws file not found Exception");
		
	}

	(expected=MalformedURLException.class)
	public void testMalformedURL() throws Exception
	{
		IXMLReader r=target.fileReader("C:/Users/zayin/Desktop/test.xml");
		fail("should throw MalFormed URL EXCEPTION");
	} */
	
	
}
