package net.n3.nanoxml;



import junit.framework.TestCase;

import java.util.Vector;



public class XMLElementTest extends TestCase {

	public void testNullFieldValues() throws Exception
	{
        XMLElement target=new XMLElement();
		String fullName=(String)NanoXMLUtil.getField(target,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(target,"namespace");
		String systemId=(String)NanoXMLUtil.getField(target,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(target,"lineNr");
		assertNull(fullName);
		assertNull(nameSpace);
		assertNull(systemId);
		assertEquals(-1,lineNumber);
	}
	
	
	public void testFieldValueFullName() throws Exception
	{
        XMLElement target=new XMLElement("note");
		String fullName=(String)NanoXMLUtil.getField(target,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(target,"namespace");
		String systemId=(String)NanoXMLUtil.getField(target,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(target,"lineNr");
		assertEquals("note",fullName);
		assertNull(nameSpace);
		assertNull(systemId);
		assertEquals(-1,lineNumber);
	}
	
	
	public void testFullNameSystemIDLineNum() throws Exception
	{
        XMLElement target=new XMLElement("note","insert.xml",1);
		String fullName=(String)NanoXMLUtil.getField(target,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(target,"namespace");
		String systemId=(String)NanoXMLUtil.getField(target,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(target,"lineNr");
		assertEquals("note",fullName);
		assertNull(nameSpace);
		assertEquals("insert.xml",systemId);
		assertEquals(1,lineNumber);
	}
	
	
	public void testFullNameandNameSpace() throws Exception
	{
        XMLElement target=new XMLElement("note","http://www.w3.org/TR/html4/");
		String fullName=(String)NanoXMLUtil.getField(target,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(target,"namespace");
		String systemId=(String)NanoXMLUtil.getField(target,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(target,"lineNr");
		assertEquals("note",fullName);
		assertNull(systemId);
		assertEquals("http://www.w3.org/TR/html4/",nameSpace);
		assertEquals(-1,lineNumber);
	}
	
	
	public void testValidFieldValues() throws Exception
	{
        XMLElement target=new XMLElement("f:note","http://www.w3.org/TR/html4/","insert.xml",2);
		String fullName=(String)NanoXMLUtil.getField(target,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(target,"namespace");
		String systemId=(String)NanoXMLUtil.getField(target,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(target,"lineNr");
		assertEquals("f:note",fullName);
		assertEquals("insert.xml",systemId);
		assertEquals("http://www.w3.org/TR/html4/",nameSpace);
		assertEquals(2,lineNumber);
	}
	
	public void testCreatePCDataElement() throws Exception
	{
        XMLElement target=new XMLElement();
		IXMLElement ele=target.createPCDataElement();
		String fullName=(String)NanoXMLUtil.getField(ele,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(ele,"namespace");
		String systemId=(String)NanoXMLUtil.getField(ele,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(ele,"lineNr");
		assertNull(fullName);
		assertNull(nameSpace);
		assertNull(systemId);
		assertEquals(-1,lineNumber);
	}
	
	
	public void testCreateElementFullName() throws Exception
	{
        XMLElement target=new XMLElement();
		IXMLElement ele=target.createElement("book");
		String fullName=(String)NanoXMLUtil.getField(ele,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(ele,"namespace");
		String systemId=(String)NanoXMLUtil.getField(ele,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(ele,"lineNr");
		assertEquals("book",fullName);
		assertNull(nameSpace);
		assertNull(systemId);
		assertEquals(-1,lineNumber);
	}
	
	
	public void testCreateElementFullNameandNameSpace() throws Exception
	{
        XMLElement target=new XMLElement();
		IXMLElement ele=target.createElement("note","http://www.w3.org/TR/html4/");
		String fullName=(String)NanoXMLUtil.getField(ele,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(ele,"namespace");
		String systemId=(String)NanoXMLUtil.getField(ele,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(ele,"lineNr");
		assertEquals("note",fullName);
		assertNull(systemId);
		assertEquals("http://www.w3.org/TR/html4/",nameSpace);
		assertEquals(-1,lineNumber);
	}
	
	
	public void testCreateElementFullNameSystemIDLineNum() throws Exception
	{
        XMLElement target=new XMLElement();
		IXMLElement ele=target.createElement("note","insert.xml",1);
		String fullName=(String)NanoXMLUtil.getField(ele,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(ele,"namespace");
		String systemId=(String)NanoXMLUtil.getField(ele,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(ele,"lineNr");
		assertEquals("note",fullName);
		assertNull(nameSpace);
		assertEquals("insert.xml",systemId);
		assertEquals(1,lineNumber);
	}
	
	
	public void testCreateElementAll() throws Exception
	{
        XMLElement target=new XMLElement();
		IXMLElement ele=target.createElement("f:note","http://www.w3.org/TR/html4/","insert.xml",2);
		String fullName=(String)NanoXMLUtil.getField(ele,"fullName");
		String nameSpace=(String)NanoXMLUtil.getField(ele,"namespace");
		String systemId=(String)NanoXMLUtil.getField(ele,"systemID");
		int lineNumber=(Integer)NanoXMLUtil.getField(ele,"lineNr");
		assertEquals("f:note",fullName);
		assertEquals("insert.xml",systemId);
		assertEquals("http://www.w3.org/TR/html4/",nameSpace);
		assertEquals(2,lineNumber);
		String root=(String)NanoXMLUtil.getField(ele,"parent");
		assertNull(root);
		
	}
	
	
	public void testAllGetMethods()
	{
        XMLElement target=new XMLElement();
		IXMLElement ele=target.createElement("f:note","http://www.w3.org/TR/html4/","insert.xml",2);
		assertEquals("note",ele.getName());
		assertEquals("http://www.w3.org/TR/html4/",ele.getNamespace());
		assertEquals("insert.xml",ele.getSystemID());
		assertEquals(2,ele.getLineNr());
	}
	
	
	public void testsetName() throws Exception
	{
        XMLElement target=new XMLElement();
		target.setName("book");
		String name=(String)NanoXMLUtil.getField(target,"name");
		String fullName=(String)NanoXMLUtil.getField(target,"fullName");
		String namespace=(String)NanoXMLUtil.getField(target,"namespace");
		assertEquals("book",name);
		assertEquals("book",fullName);
		assertNull(namespace);
	}
	
	
	public void testsetNameWithArgs() throws Exception
	{
        XMLElement target=new XMLElement();
		target.setName("f:book","http://www.w3.org/TR/html4/");
		String name=(String)NanoXMLUtil.getField(target,"name");
		String fullName=(String)NanoXMLUtil.getField(target,"fullName");
		String namespace=(String)NanoXMLUtil.getField(target,"namespace");
		assertEquals("book",name);
		assertEquals("f:book",fullName);
		assertEquals("http://www.w3.org/TR/html4/",namespace);
		
		target.setName("book","http://www.w3.org/TR/html4/");
		name=(String)NanoXMLUtil.getField(target,"name");
		fullName=(String)NanoXMLUtil.getField(target,"fullName");
		namespace=(String)NanoXMLUtil.getField(target,"namespace");
		assertEquals("book",name);
		assertEquals("book",fullName);
		assertEquals("http://www.w3.org/TR/html4/",namespace);
	}
	
	
	public void testAddChild()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("to","insert.xml",2);
		target.addChild(child);
		assertEquals("note",child.getParent().getName());
		assertEquals(1,target.getChildrenCount());
	}
	
	//(expected=IllegalArgumentException.class)
	public void testAddChildNull()
	{
		//target=new XMLElement("note","insert.xml",1);
		//target.addChild(null);
		//fail("should throw illegalArgumentException");
	}
	
	
	public void testAddChildNameNull()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("to","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement(null);
		assertNull(child2.getName());
		target.addChild(child2);
		assertEquals(2,target.getChildrenCount());
	}
	
	
	public void testLastChildNameNull()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement(null);
		target.addChild(child);
		IXMLElement child2=new XMLElement(null);
		assertNull(child2.getName());
		target.addChild(child2);
		assertEquals(1,target.getChildrenCount());
	}
	
	
	public void testInsertChild() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement("to","insert.xml",1);
		target.insertChild(child2,1);
		Vector children=(Vector)NanoXMLUtil.getField(target,"children");
		assertTrue(children.contains(child2));
		assertEquals("note",child2.getParent().getName());
		assertEquals(2,target.getChildrenCount());
		
	}
	
	
	//(expected=IllegalArgumentException.class)
	public void testInsertNullChild() throws Exception
	{
	//	target=new XMLElement("note","insert.xml",1);
	//	IXMLElement child=new XMLElement("to","insert.xml",2);
	//	target.addChild(child);
	//	IXMLElement child2=null;
	//	target.insertChild(child2,1);
		
		
	}
	
	public void testInsertChildNull() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement(null);
		target.insertChild(child,0);
		IXMLElement child2=new XMLElement(null);
		assertNull(child2.getName());
		target.insertChild(child2,1);
		Vector children=(Vector)NanoXMLUtil.getField(target,"children");
		assertNull(child.getName());
		assertNull(child2.getName());
		assertEquals(1,target.getChildrenCount());
		
	}
	
//	(expected=IllegalArgumentException.class)
	public void testRemoveNullChild() throws Exception
	{
		//target=new XMLElement("note","insert.xml",1);
		//target.removeChild(null);
	}
	
	
	public void testRemoveChild() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement("to","insert.xml",1);
		target.insertChild(child2,1);
		Vector children=(Vector)NanoXMLUtil.getField(target,"children");
		assertTrue(children.contains(child2));
		assertEquals("note",child2.getParent().getName());
		assertEquals(2,target.getChildrenCount());
		
		target.removeChild(child2);
		assertFalse(children.contains(child2));
		assertEquals(1,target.getChildrenCount());
		target.removeChild(child);
		assertFalse(children.contains(child2));
		assertEquals(0,target.getChildrenCount());
	}
	
	
	public void testRemoveChildAtIndex() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement("to","insert.xml",1);
		target.insertChild(child2,1);
		Vector children=(Vector)NanoXMLUtil.getField(target,"children");
		assertTrue(children.contains(child2));
		assertEquals("note",child2.getParent().getName());
		assertEquals(2,target.getChildrenCount());
		
		target.removeChildAtIndex(0);
		assertFalse(children.contains(child));
		assertEquals(1,target.getChildrenCount());	
		
	}
	
	
	public void testIsLeaf() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		assertTrue(target.isLeaf());
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		assertFalse(target.isLeaf());
		assertTrue(child.isLeaf());
	}
	
	
	public void testHadChildren() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		assertFalse(target.hasChildren());
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		assertTrue(target.hasChildren());
		assertFalse(child.hasChildren());
		
	}
	
	//(expected=ArrayIndexOutOfBoundsException.class)
	public void testGetChildAtIndexOutOfBound()
	{
		/*target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement("to","insert.xml",1);
		target.insertChild(child2,1);
		target.getChildAtIndex(2);
		*/
	}
	
	public void testGetChildAtIndex()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement("to","insert.xml",1);
		target.insertChild(child2,1);
		assertEquals("from",target.getChildAtIndex(0).getName());
		assertEquals("to",target.getChildAtIndex(1).getName());
	}
	
	
	public void testGetFirstChildNames()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement("to","insert.xml",1);
		target.insertChild(child2,1);
		assertEquals("from",target.getFirstChildNamed("from").getName());
		assertNull(target.getFirstChildNamed("subject"));
	}
	
	
	public void testGetFirstChildNameSpace()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("n:note","http://www.w3.org/TR/html4/","insert.xml",1);
		IXMLElement childP1=new XMLElement("n:subject","http://www.w3.org/TR/html4/","insert.xml",2);
		target.addChild(childP1);
		IXMLElement child2P1=new XMLElement("n:to","http://www.w3.org/TR/html4/","insert.xml",1);
		target.addChild(child2P1);
		
		target=new XMLElement("f:note","http://www.w3.org/TR/html5/","insert.xml",1);
		IXMLElement childP2=new XMLElement("f:from","http://www.w3.org/TR/html5/","insert.xml",2);
		target.addChild(childP2);
		IXMLElement child2P2=new XMLElement("f:to","http://www.w3.org/TR/html5/","insert.xml",1);
		target.addChild(child2P2);
		assertEquals("from",target.getFirstChildNamed("from","http://www.w3.org/TR/html5/").getName());
		assertNull(target.getFirstChildNamed("from","http://www.w3.org/TR/html4/"));
		assertNull(target.getFirstChildNamed("subject","http://www.w3.org/TR/html5/"));
	}
	
	
	public void testGetAllChildNames()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement child=new XMLElement("from","insert.xml",2);
		target.addChild(child);
		IXMLElement child2=new XMLElement("to","insert.xml",1);
		target.addChild(child2);
		IXMLElement child3=new XMLElement("to","insert.xml",3);
		target.addChild(child3);
		Vector allChildren=target.getChildrenNamed("to");
		assertEquals(2,allChildren.size());
		allChildren=target.getChildrenNamed("subject");
		assertEquals(0,allChildren.size());
		
	}
	
	public void testGellChildrenNamedNameSpace()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("f:note","http://www.w3.org/TR/html5/","insert.xml",1);
		IXMLElement childP2=new XMLElement("f:to","http://www.w3.org/TR/html5/","insert.xml",2);
		target.addChild(childP2);
		IXMLElement child2P2=new XMLElement("f:to","http://www.w3.org/TR/html5/","insert.xml",1);
		target.addChild(child2P2);
		IXMLElement child3P1=new XMLElement("n:to","http://www.w3.org/TR/html4/","insert.xml",1);
		target.addChild(child3P1);
		Vector allChildren=target.getChildrenNamed("to","http://www.w3.org/TR/html5/");
		assertEquals(2,allChildren.size());
		allChildren=target.getChildrenNamed("subject","http://www.w3.org/TR/html5/");
		assertEquals(0,allChildren.size());
	}
	
	
	public void testGetSetAttribute()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement childP2=new XMLElement("to","insert.xml",2);
		childP2.setAttribute("type","book");
		target.addChild(childP2);
		target.setAttribute("address","email");
		String attributeValue=target.getAttribute("address");
		assertEquals("email",attributeValue);
		attributeValue=childP2.getAttribute("type");
		assertEquals("book",attributeValue);
	}
	
	
	public void testNullAttribute()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement childP2=new XMLElement("to","insert.xml",2);
		assertEquals("book",childP2.getAttribute(null,"book"));
	}
	
	
	public void testGetSetAttributeNameSpace()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("n:note","http://www.w3.org/TR/html4/","insert.xml",1);
		IXMLElement childP2=new XMLElement("f:to","http://www.w3.org/TR/html5/","insert.xml",2);
		childP2.setAttribute("type","http://www.w3.org/TR/html5/","book");
		target.addChild(childP2);
		target.setAttribute("type","http://www.w3.org/TR/html4/","email");
		String attributeValue=target.getAttribute("type","http://www.w3.org/TR/html4/","post");
		assertEquals("email",attributeValue);
		attributeValue=childP2.getAttribute("type","http://www.w3.org/TR/html5/","notebook");
		
		assertEquals("book",attributeValue);
		assertEquals("CDATA",target.getAttributeType("type"));
		assertEquals("http://www.w3.org/TR/html4/",target.getAttributeNamespace("type"));
		assertEquals("CDATA",target.getAttributeType("type","http://www.w3.org/TR/html4/"));
		
	}
	
	
	public void testNullAttributeNameSpace()
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","http://www.w3.org/TR/html4/","insert.xml",1);
		IXMLElement childP2=new XMLElement("n:to","http://www.w3.org/TR/html4/","insert.xml",2);
		assertEquals("book",childP2.getAttribute(null,"http://www.w3.org/TR/html4/","book"));
		assertNull(target.getAttributeType("type"));
		assertNull(target.getAttributeNamespace("type"));
		assertNull(target.getAttributeType("type","http://www.w3.org/TR/html4/"));
	}
	
	
	public void testRemoveAttribute() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement childP2=new XMLElement("to","insert.xml",2);
		childP2.setAttribute("type","book");
		target.addChild(childP2);
		target.setAttribute("address","email");
		String attributeValue=target.getAttribute("address");
		assertEquals("email",attributeValue);
		attributeValue=childP2.getAttribute("type");
		assertEquals("book",attributeValue);
		
		Vector attributes=(Vector)NanoXMLUtil.getField(target,"attributes");
		assertEquals(1,attributes.size());
		assertTrue(target.hasAttribute("address"));
		target.removeAttribute("address");		
		target.removeAttribute("email");
		assertEquals(0,attributes.size());
		assertFalse(target.hasAttribute("address"));
	}
	
	
	public void testRemoveAttributeNameSpace() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("n:note","http://www.w3.org/TR/html4/","insert.xml",1);
		IXMLElement childP2=new XMLElement("f:to","http://www.w3.org/TR/html5/","insert.xml",2);
		childP2.setAttribute("type","http://www.w3.org/TR/html5/","book");
		target.addChild(childP2);
		target.setAttribute("type","http://www.w3.org/TR/html4/","email");
		String attributeValue=target.getAttribute("type","http://www.w3.org/TR/html4/","post");
		assertEquals("email",attributeValue);
		attributeValue=childP2.getAttribute("type","http://www.w3.org/TR/html5/","notebook");
		
		Vector attributes=(Vector)NanoXMLUtil.getField(target,"attributes");
		assertEquals(1,attributes.size());
		assertTrue(target.hasAttribute("type","http://www.w3.org/TR/html4/"));
		target.removeAttribute("type","http://www.w3.org/TR/html4/");
		
		target.removeAttribute("email","http://www.w3.org/TR/html4/");
		assertEquals(0,attributes.size());
		assertFalse(target.hasAttribute("type","http://www.w3.org/TR/html4/"));
	}
	
	
	public void testNotEqualsXMLElement() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement childTarget=new XMLElement("to","insert.xml",1);
		target.addChild(childTarget);
		IXMLElement target1=new XMLElement("note","insert.xml",1);
		target.setAttribute("type","email");
		target.setAttribute("address","post");
		target1.setAttribute("type","email");
		target1.setAttribute("address","post");
		target1.setAttribute("phone","123456");
		assertFalse(target.equalsXMLElement(target1));
	}
	
	
	public void testEqualsXMLElement() throws Exception
	{
        XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		IXMLElement childTarget=new XMLElement("to","insert.xml",1);
		target.addChild(childTarget);
		IXMLElement target1=new XMLElement("note","insert.xml",1);
		IXMLElement childTarget1=new XMLElement("to","insert.xml",1);
		target1.addChild(childTarget1);
		target.setAttribute("type","email");
		target.setAttribute("address","post");
		target1.setAttribute("type","email");
		target1.setAttribute("address","post");
		assertTrue(target.equalsXMLElement(target1));
	}
	
	
	public void testEqualsThrowException()
	{   XMLElement target=new XMLElement();
		target=new XMLElement("note","insert.xml",1);
		String target1="new Element";
		assertFalse(target1.equals(target));
				
	}
	
	
	
	
	
}
