package net.n3.nanoxml;

import junit.framework.TestCase;

import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.util.Hashtable;



public class XMLEntityResolverTest extends TestCase {
	//private XMLEntityResolver target=null;

	
	
	public void testEntrySize() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		Hashtable entries=(Hashtable)NanoXMLUtil.getField(target,"entities");
		assertEquals(5,entries.size());
	}
	
	public void testAddInternalEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		target.addInternalEntity("hash","&#35;");
		Hashtable entries=(Hashtable)NanoXMLUtil.getField(target,"entities");
		assertEquals(6,entries.size());
	}
	
	public void testBadAddInternalEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		target.addInternalEntity("amp","&#35;");
		Hashtable entries=(Hashtable)NanoXMLUtil.getField(target,"entities");
		assertEquals(5,entries.size());
	}
	
	
	public void testAddExternalEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		target.addExternalEntity("externalEntity",null,"file:sample.xml");
		Hashtable entries=(Hashtable)NanoXMLUtil.getField(target,"entities");
		assertEquals(6,entries.size());
	}
	
	
	public void testBadAddExternalEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		target.addExternalEntity("externalEntity",null,"file:sample.xml");
		Hashtable entries=(Hashtable)NanoXMLUtil.getField(target,"entities");
		assertEquals(6,entries.size());
		target.addExternalEntity("externalEntity","publicID","file:sample.xml");
		assertEquals(6,entries.size());
		
	}
	
	
	public void testGetEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		IXMLReader reader = StdXMLReader.fileReader("src/test/resources/test.xml");
		Reader result=target.getEntity(reader,"amp");
		assertTrue(result instanceof StringReader);
	}
	
	
	public void testNullObjectGetEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		IXMLReader reader = StdXMLReader.fileReader("src/test/resources/test.xml");
		Reader result=target.getEntity(reader,"hash");
		assertNull(result);
	}
	
	public void testReturnExternalEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		target.addExternalEntity("externalEntity",null,"file:src/test/resources/test.xml");
		IXMLReader reader = StdXMLReader.fileReader("src/test/resources/test.xml");
		Reader result=target.getEntity(reader,"externalEntity");
		assertTrue(result instanceof InputStreamReader);
	}
	
	
	public void testIsNotExternalEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		assertFalse(target.isExternalEntity("amp"));
	}
	
	
	public void testIsExternalEntity() throws Exception
	{
        XMLEntityResolver target=new XMLEntityResolver();
		target.addExternalEntity("externalEntity",null,"file:test.xml");
		assertTrue(target.isExternalEntity("externalEntity"));
	}

    /*
	public void testThrowException() throws Exception
	{
		target.addExternalEntity("externalEntity",null,"file:sample.xml");
		IXMLReader reader = StdXMLReader.fileReader("TestInputFiles/test.xml");
		Reader result=target.getEntity(reader,"externalEntity");
		assertTrue(result instanceof InputStreamReader);
	
	} */
}
