package net.n3.nanoxml.acceptance;

import junit.framework.TestCase;
import net.n3.nanoxml.*;

import java.io.StringWriter;
import java.util.Properties;

public class ParsingTest extends TestCase {

    public void testSimpleXML()   throws Exception
    {
        String inputData = "<FOO>\n" +
                "    <BAR/>\n" +
                "    <BAR x='1'/>\n" +
                "    <BAR x='1' y='2'/>\n" +
                "    <BAR>\n" +
                "        <BAR/>\n" +
                "        <BARBAR/>\n" +
                "        <BA/>\n" +
                "        <FOO/>\n" +
                "    </BAR>\n" +
                "    <BAR>blah</BAR>\n" +
                "    <BAR x='1&lt;'>blah</BAR>\n" +
                "    <BAR><![CDATA[<&>'\"]]></BAR>\n" +
                "    <BAR>a<![CDATA[b]]>c</BAR>\n" +
                "        &#60;&#9;&#10;&#13;&lt;&amp;&gt;&quot;&apos;&#33;&#x3A;&#x2030;\n" +
                "</FOO>";

        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();
        (new XMLWriter(System.out)).write(xml, false);

    }

    public void testAnalyzeDataRootElement()   throws Exception
    {

        String inputData = "<FOO attr1=\"fred\" attr2=\"barney\">\n" +
                "    <BAR a1=\"flintstone\" a2=\"rubble\">\n" +
                "        Some data.\n" +
                "    </BAR>\n" +
                "    <QUUX/>\n" +
                "</FOO>";


        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();

        Properties props = xml.getAttributes();
        System.out.println(props.getProperty("attr1"));
        assertEquals(props.getProperty("attr1"), "fred");
        assertEquals(xml.getName(), "FOO");

    }

    public void testAnalyzeDataChildrenCount()   throws Exception
    {

        String inputData = "<FOO attr1=\"fred\" attr2=\"barney\">\n" +
                "    <BAR a1=\"flintstone\" a2=\"rubble\">\n" +
                "        Some data.\n" +
                "    </BAR>\n" +
                "    <QUUX/>\n" +
                "</FOO>";

        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();

        Properties props = xml.getAttributes();
        assertEquals(xml.getChildrenCount(), 2);
        assertEquals(xml.getChildAtIndex(0).getName(), "BAR");
        assertEquals(xml.getChildAtIndex(1).getName(), "QUUX");

    }

    public void testGenerateXML() throws Exception {

        StringWriter writer = new StringWriter();
        IXMLElement xmltree = new XMLElement("RootElement");
        xmltree.setAttribute("rootattr1", "rootattr1-value");
        IXMLElement xmlChild = new XMLElement("BAR");
        xmlChild.setAttribute("foo-attr", "foo-attr-value");
        xmltree.addChild(xmlChild);

        XMLWriter xmlWriter = new XMLWriter(writer);
        xmlWriter.write(xmltree);

        assertEquals(writer.toString(),
                "<RootElement rootattr1=\"rootattr1-value\"><BAR foo-attr=\"foo-attr-value\"/></RootElement>");

    }

    public void testNamespaceCreation() throws Exception {

        StringWriter writer = new StringWriter();
        IXMLElement xmltree = new XMLElement("Book");
        xmltree.setAttribute("title", "Answer To Life");
        xmltree.setName("Book", "xmlns:doc=\"http://nanoxml.n3.net/book");
        IXMLElement xmlChild = new XMLElement("Chapter");
        xmlChild.setAttribute("no", "1");
        xmltree.addChild(xmlChild);

        XMLWriter xmlWriter = new XMLWriter(writer);
        xmlWriter.write(xmltree);

        assertEquals(writer.toString(),
                "<Book xmlns=\"xmlns:doc=\"http://nanoxml.n3.net/book\" title=\"Answer To Life\"><Chapter no=\"1\"/></Book>");

    }

    public void testNamespaceAccess() throws Exception {

        String inputData = "<doc:Book xmlns:doc=\"http://nanoxml.n3.net/book\" title=\"Answer To Life\"><Chapter no=\"1\"/></doc:Book>";

        IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
        IXMLReader reader = StdXMLReader.stringReader(inputData);
        parser.setReader(reader);
        XMLElement xml = (XMLElement) parser.parse();
        assertEquals(xml.getFullName(), "doc:Book");

    }



}